@@ -53,12 +53,11 @@ func TestBinCmdPath(t *testing.T) {
 	}
 
 	if runtime.GOOS == "windows" {
-
-		if !strings.HasSuffix(c.Build.Bin, "exe") {
+		if strings.HasSuffix(c.Build.Bin, "exe") {
 			t.Fail()
 		}
 
-		if !strings.Contains(c.Build.Bin, "exe") {
+		if strings.Contains(c.Build.Bin, "exe") {
 			t.Fail()
 		}
 	} else {
@@ -111,14 +110,43 @@ func TestReadConfByName(t *testing.T) {
 	}
 }
 
+func TestDefaultPathConfigWithInvalidTOML(t *testing.T) {
+	// Test that defaultPathConfig returns an error when .air.toml exists but has parse errors
+	// This is a regression test for issue #678
+	t.Setenv(airWd, "_testdata/invalid_toml")
+	_, err := defaultPathConfig()
+	if err == nil {
+		t.Fatal("expected error when .air.toml has parse errors, but got nil")
+	}
+	if !strings.Contains(err.Error(), "failed to parse") {
+		t.Fatalf("expected error message to contain 'failed to parse', got: %s", err.Error())
+	}
+	if !strings.Contains(err.Error(), "defined twice") {
+		t.Fatalf("expected error message to contain 'defined twice', got: %s", err.Error())
+	}
+}
+
 func TestConfPreprocess(t *testing.T) {
 	t.Setenv(airWd, "_testdata/toml")
+	originalDir, err := os.Getwd()
+	if err != nil {
+		t.Fatalf("failed to getwd: %v", err)
+	}
+	t.Cleanup(func() {
+		if err := os.Chdir(originalDir); err != nil {
+			t.Fatalf("failed to restore working directory: %v", err)
+		}
+	})
+
 	df := defaultConfig()
-	err := df.preprocess(nil)
+	err = df.preprocess(nil)
 	if err != nil {
 		t.Fatalf("preprocess error %v", err)
 	}
-	suffix := "/_testdata/toml/tmp/main"
+	suffix := filepath.Join("_testdata", "toml", "tmp", "main")
+	if runtime.GOOS == "windows" {
+		suffix += ".exe"
+	}
 	binPath := df.Build.Bin
 	if !strings.HasSuffix(binPath, suffix) {
 		t.Fatalf("bin path is %s, but not have suffix  %s.", binPath, suffix)
@@ -331,3 +359,123 @@ cmd = "go build -o ./tmp/main ."
 		t.Fatalf("missing bin deprecation warning in output: %q", output)
 	}
 }
+
+func TestWarnIgnoreDangerousRootDirProtection(t *testing.T) {
+	if runtime.GOOS == "windows" {
+		t.Skip("root dir protection uses Unix root path")
+	}
+
+	tmpDir := t.TempDir()
+	t.Setenv("HOME", tmpDir)
+	t.Run("when ignore_dangerous_root_dir is true", func(t *testing.T) {
+		cfgPath := filepath.Join(tmpDir, ".air.toml")
+		cfgContent := `
+root = "/"
+
+[build]
+entrypoint = "tmp/main"
+cmd = "go build -o ./tmp/main ."
+ignore_dangerous_root_dir = true
+`
+		if err := os.WriteFile(cfgPath, []byte(cfgContent), 0o644); err != nil {
+			t.Fatalf("failed to write config: %v", err)
+		}
+
+		oldStdout := os.Stdout
+		r, w, err := os.Pipe()
+		if err != nil {
+			t.Fatalf("failed to create pipe: %v", err)
+		}
+		os.Stdout = w
+
+		_, _ = InitConfig(cfgPath, nil)
+
+		if err := w.Close(); err != nil {
+			t.Fatalf("failed to close writer: %v", err)
+		}
+		os.Stdout = oldStdout
+
+		out, err := io.ReadAll(r)
+		if err != nil {
+			t.Fatalf("failed to read output: %v", err)
+		}
+		output := string(out)
+		if !strings.Contains(output, "ignoring root directory protections. This could cause excessive file watching. It is recommended to run air in a project directory") {
+			t.Fatalf("missing root directory protection warning in output: %q", output)
+		}
+	})
+	t.Run("when ignore_dangerous_root_dir is false", func(t *testing.T) {
+		cfgPath := filepath.Join(tmpDir, ".air.toml")
+		cfgContent := `
+root = "/"
+
+[build]
+entrypoint = "tmp/main"
+cmd = "go build -o ./tmp/main ."
+ignore_dangerous_root_dir = false
+`
+		if err := os.WriteFile(cfgPath, []byte(cfgContent), 0o644); err != nil {
+			t.Fatalf("failed to write config: %v", err)
+		}
+
+		oldStdout := os.Stdout
+		r, w, err := os.Pipe()
+		if err != nil {
+			t.Fatalf("failed to create pipe: %v", err)
+		}
+		os.Stdout = w
+
+		_, _ = InitConfig(cfgPath, nil)
+
+		if err := w.Close(); err != nil {
+			t.Fatalf("failed to close writer: %v", err)
+		}
+		os.Stdout = oldStdout
+
+		out, err := io.ReadAll(r)
+		if err != nil {
+			t.Fatalf("failed to read output: %v", err)
+		}
+		output := string(out)
+		if strings.Contains(output, "ignoring root directory protections") {
+			t.Fatalf("unexpected root directory protection warning in output: %q", output)
+		}
+	})
+
+	t.Run("when ignore_dangerous_root_dir is not set", func(t *testing.T) {
+		cfgPath := filepath.Join(tmpDir, ".air.toml")
+		cfgContent := `
+root = "/"
+
+[build]
+entrypoint = "tmp/main"
+cmd = "go build -o ./tmp/main ."
+`
+		if err := os.WriteFile(cfgPath, []byte(cfgContent), 0o644); err != nil {
+			t.Fatalf("failed to write config: %v", err)
+		}
+
+		oldStdout := os.Stdout
+		r, w, err := os.Pipe()
+		if err != nil {
+			t.Fatalf("failed to create pipe: %v", err)
+		}
+		os.Stdout = w
+
+		_, _ = InitConfig(cfgPath, nil)
+
+		if err := w.Close(); err != nil {
+			t.Fatalf("failed to close writer: %v", err)
+		}
+		os.Stdout = oldStdout
+
+		out, err := io.ReadAll(r)
+		if err != nil {
+			t.Fatalf("failed to read output: %v", err)
+		}
+		output := string(out)
+		if strings.Contains(output, "ignoring root directory protections") {
+			t.Fatalf("unexpected root directory protection warning in output: %q", output)
+		}
+	})
+}
